"""Module with functions for 'deploy' subpackage."""

from __future__ import annotations
import os
import shutil

import mylogging

from mypythontools.paths import validate_path, PathLike
from mypythontools.misc import check_script_is_available, delete_files
from mypythontools.terminal import terminal_do_command, PYTHON

from mypythontools_cicd.project_paths import PROJECT_PATHS


def deploy_to_pypi(
    setup_path: None | PathLike = None, clean: bool = True, verbose: bool = True, pep517: bool = True
) -> None:
    """Publish python library to PyPi.

    Username and password are set with env vars `TWINE_USERNAME` and `TWINE_PASSWORD`.

    Note:
        You need working `setup.py` file. If you want to see example, try the one from project-starter on

        https://github.com/Malachov/mypythontools/blob/master/content/project-starter/setup.py

    Args:
        setup_path (None | PathLike, optional): Function suppose, that there is a setup.py somewhere in cwd.
            If not, path will be inferred. Build and dist folders will be created in same directory.
            Defaults to None.
        clean (bool, optional): Whether delete created build and dist folders.
        verbose (bool, optional): If True, result of terminal command will be printed to console.
            Defaults to False.
        pep517 (bool, optional): Whether using PEP 517, that use pyproject.toml to build distribution.
    """
    usr = os.environ.get("TWINE_USERNAME")
    password = os.environ.get("TWINE_PASSWORD")

    if not usr or not password:
        raise KeyError("Setup env vars TWINE_USERNAME and TWINE_PASSWORD to use deploy.")

    check_script_is_available("twine", "twine")

    setup_path = PROJECT_PATHS.root / "setup.py" if not setup_path else validate_path(setup_path)

    setup_dir_path = setup_path.parent

    dist_path = setup_dir_path / "dist"
    build_path = setup_dir_path / "build"

    delete_files(dist_path)
    delete_files(build_path)

    if pep517:
        build_command = f"{PYTHON} -m build --wheel --sdist"
    else:
        build_command = f"{PYTHON} setup.py sdist bdist_wheel"

    terminal_do_command(
        build_command,
        cwd=setup_dir_path.as_posix(),
        verbose=verbose,
        error_header="Build python distribution for PyPi deployment failed. Try to set 'pep517' to False.",
    )

    command = f"twine upload dist/*"

    terminal_do_command(
        command,
        cwd=setup_dir_path.as_posix(),
        verbose=verbose,
        error_header="Deploying to PyPi failed.",
    )

    if clean:
        delete_files(dist_path)
        delete_files(build_path)



"""Module with functions for 'misc' subpackage."""

from __future__ import annotations
from typing import Sequence
from pathlib import Path
import os

from typing_extensions import Literal
from mypythontools.paths import validate_path, PathLike

from mypythontools_cicd.project_paths import PROJECT_PATHS


def get_requirements_files(requirements: Literal["infer"] | PathLike | Sequence[PathLike]) -> list[Path]:
    """Consolidate various input types into defined one.

    Args:
        requirements (Literal["infer"] | PathLike | Sequence[PathLike]): E.g. ["requirements.txt",
            "requirements_dev.txt"]. if 'infer', then every file where requirements is in the name is used.

    Returns:
        list[Path]: List of paths to requirements files.
    """

    if requirements == "infer":

        requirements_parsed = []

        for i in PROJECT_PATHS.root.glob("*"):
            if "requirements" in i.as_posix().lower() and i.suffix == ".txt":
                requirements_parsed.append(i)
    else:
        if isinstance(requirements, (Path, str, os.PathLike)):
            requirements = [requirements]

        requirements_parsed = [validate_path(req) for req in requirements]

    return requirements_parsed



"""Module with functions for 'venvs' subpackage."""

from __future__ import annotations
from typing import Sequence
import platform
import subprocess
import shutil
from pathlib import Path
from ast import literal_eval
import sys

from typing_extensions import Literal

from mypythontools.paths import PathLike
from mypythontools.terminal import (
    get_console_str_with_quotes,
    terminal_do_command,
    SHELL_AND,
    PYTHON,
    EXECUTABLE,
)

from ..misc import get_requirements_files
from mypythontools_cicd.project_paths import PROJECT_PATHS


class Venv:
    """You can create new venv or sync it's dependencies.

    Example:
        >>> from pathlib import Path
        ...
        >>> path = "venv/310" if platform.system() == "Windows" else "venv/ubuntu"
        >>> venv = Venv(path)
        >>> venv.create()  # If already exists, it's skipped
        >>> venv.install_library("colorama==0.3.9")
        >>> "colorama==0.3.9" in venv.list_packages()
        True
        >>> venv.sync_requirements()  # There ia a 0.4.4 in requirements.txt
        >>> "colorama==0.4.4" in venv.list_packages()
        True
        >>> venv.remove()
        >>> Path(path).exists()
        False
    """

    def __init__(self, venv_path: PathLike) -> None:
        """Init the venv class. To create or update it, you can call extra functions.

        Args:
            venv_path(PathLike): Path of venv. E.g. `venv`

        Raises:
            FileNotFoundError: No folder found on defined path.
        """
        self.venv_path = Path(venv_path).resolve()
        """Path to venv prefix, e.g. .../venv"""

        if not self.venv_path.exists():
            self.venv_path.mkdir()

        self.venv_path_console_str = get_console_str_with_quotes(self.venv_path)

        if not self.venv_path.exists():
            self.venv_path.mkdir(parents=True, exist_ok=True)

        if platform.system() == "Windows":
            activate_path = self.venv_path / "Scripts" / "activate.bat"
            self.executable = self.venv_path / "Scripts" / "python.exe"
            self.executable_str = get_console_str_with_quotes(
                (self.venv_path / "Scripts" / "python.exe").as_posix()
            )
            self.create_command = f"python -m virtualenv {self.venv_path_console_str}"
            self.activate_command = get_console_str_with_quotes(activate_path.as_posix())
            scripts_path = self.venv_path / "Scripts"
        else:
            self.executable = self.venv_path / "bin" / "python"
            self.executable_str = get_console_str_with_quotes((self.venv_path / "bin" / "python").as_posix())
            self.create_command = f"python3 -m virtualenv {self.venv_path_console_str}"
            self.activate_command = (
                f"source {get_console_str_with_quotes(self.venv_path / 'bin' / 'activate')}"
            )
            scripts_path = self.venv_path / "bin"

        self.installed = True if (self.executable).exists() else False

        self.scripts_path: Path = scripts_path
        """Path to the executables. Can be directly used in terminal. Some libraries cannot use
        ``python -m package`` syntax and therefore it can be called from scripts folder."""

        self.subprocess_prefix = f"{self.activate_command} {SHELL_AND} {self.executable_str} -m "
        """Run as module, so library can be directly call afterwards. Can be directly used in terminal. It can
        look like this for example::
        
            .../Scripts/activate.bat && .../venv/Scripts/python.exe -m
        """

    def create(self, verbose: bool = False) -> None:
        """Create virtual environment. If it already exists, it will be skipped and nothing happens.

        Args:
            verbose (bool, optional): If True, result of terminal command will be printed to console.
                Defaults to False.
        """
        if not self.installed:
            terminal_do_command(
                self.create_command,
                cwd=PROJECT_PATHS.root.as_posix(),
                shell=True,
                verbose=verbose,
                error_header="Venv creation failed",
            )

    def sync_requirements(
        self, requirements: Literal["infer"] | PathLike | Sequence[PathLike] = "infer", verbose: bool = False
    ) -> None:
        """Sync libraries based on requirements. Install missing, remove unnecessary.

        Args:
            requirements (Literal["infer"] | PathLike | Sequence[PathLike], optional): Define what libraries
                will be installed. If "infer", autodetected. Can also be a list of more files e.g
                `["requirements.txt", "requirements_dev.txt"]`. Defaults to "infer".
            verbose (bool, optional): If True, result of terminal command will be printed to console.
                Defaults to False.
        """
        requirements = get_requirements_files(requirements)

        requirements_content = ""

        for i in requirements:
            with open(i, "r") as req:
                requirements_content = requirements_content + "\n" + req.read()

        # requirements_content = f"{requirements_content}\nmypythontools\npytest"

        requirements_all_path = self.venv_path / "requirements_all.in"
        requirements_all_console_path_str = get_console_str_with_quotes(requirements_all_path)
        freezed_requirements_console_path_str = get_console_str_with_quotes(
            self.venv_path / "requirements.txt"
        )

        with open(requirements_all_path, "w") as requirement_libraries:
            requirement_libraries.write(requirements_content)

        pip_compile_command = (
            f"pip-compile {requirements_all_console_path_str} --output-file "
            f"{freezed_requirements_console_path_str} --quiet"
        )

        pip_tools_install_msg = "Library necessary for syncing requirements 'pip-tools' failed to install."

        sync_commands = {
            "pip install pip-tools": pip_tools_install_msg,
            pip_compile_command: "Creading joined requirements.txt file failed.",
            f"pip-sync {freezed_requirements_console_path_str} --quiet": "Requirements syncing failed.",
        }

        for i, j in sync_commands.items():
            terminal_do_command(
                f"{self.activate_command} {SHELL_AND} {i}", shell=True, verbose=verbose, error_header=j
            )

    def list_packages(self) -> str:
        """Get list of installed libraries in the venv.

        The reason why it's meta coded via string parsing and not parsed directly is that it needs to be
        available from other venv as well.
        """
        result = subprocess.run(
            f"{self.activate_command} {SHELL_AND} {PYTHON} -m pip freeze",
            capture_output=True,
            check=True,
            shell=True,
            executable=EXECUTABLE,
        )
        output_str = result.stdout.decode().strip("\r\n")

        return output_str

    def install_library(self, name: str, verbose: bool = False) -> None:
        """Install package to venv with pip install.

        Args:
            verbose (bool, optional): If True, result of terminal command will be printed to console.
                Defaults to False.
        """
        command = f"{self.activate_command} {SHELL_AND} pip install {name}"
        terminal_do_command(command, shell=True, verbose=verbose, error_header="Library installation failed.")

    def uninstall_library(self, name: str, verbose: bool = False) -> None:
        """Uninstall package to venv with pip install.

        Args:
            verbose (bool, optional): If True, result of terminal command will be printed to console.
                Defaults to False.
        """
        command = f"{self.activate_command} {SHELL_AND} pip uninstall {name}"

        terminal_do_command(command, shell=True, verbose=verbose, error_header="Library removal failed")

    def remove(self) -> None:
        """Remove the folder with venv."""
        shutil.rmtree(self.venv_path.as_posix())

    def get_script_path(self, name: str) -> str:
        """Get script path such as pip for example."""
        if platform.system() == "Windows":
            return get_console_str_with_quotes(self.scripts_path / f"{name}.exe")
        else:
            return get_console_str_with_quotes(self.scripts_path / name)


def is_venv() -> bool:
    """True if run in venv, False if run in main interpreter directly."""
    return sys.base_prefix.startswith(sys.prefix)



"""Module with functions for 'tests' subpackage."""

from __future__ import annotations
from typing import Sequence, cast
from pathlib import Path
import sys
import warnings
import os

from typing_extensions import Literal
import numpy as np
import mylogging


from mypythontools.paths import validate_path, PathLike
from mypythontools.terminal import get_console_str_with_quotes, terminal_do_command
from mypythontools.misc import delete_files

from .. import venvs
from ..misc import get_requirements_files
from mypythontools_cicd.project_paths import PROJECT_PATHS


def setup_tests(
    generate_readme_tests: bool = True,
    matplotlib_test_backend: bool = False,
    set_numpy_random_seed: int | None = 2,
) -> None:
    """Add paths to be able to import local version of library as well as other test files.

    Value Mylogging.config.colorize = 0 changed globally.

    Note:
        Function expect `tests` folder on root. If not, test folder will not be added to sys path and
        imports from tests will not work.

    Args:
        generate_readme_tests (bool, optional): If True, generete new tests from readme if there are
            new changes. Defaults to True.
        matplotlib_test_backend (bool, optional): If using matlplotlib, it need to be
            closed to continue tests. Change backend to agg. Defaults to False.
        set_numpy_random_seed (int | None): If using numpy random numbers, it will be each time the same.
            Defaults to 2.

    """
    mylogging.config.colorize = False

    PROJECT_PATHS.add_root_to_sys_path()

    # Find paths and add to sys.path to be able to import local modules
    test_dir_path = PROJECT_PATHS.tests

    if test_dir_path.as_posix() not in sys.path:
        sys.path.insert(0, test_dir_path.as_posix())

    if matplotlib_test_backend:
        import matplotlib

        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            matplotlib.use("agg")

    if generate_readme_tests:
        add_readme_tests()

    if set_numpy_random_seed:
        np.random.seed(2)


def run_tests(
    tested_path: None | PathLike = None,
    tests_path: None | PathLike = None,
    test_coverage: bool = True,
    stop_on_first_error: bool = True,
    virtualenvs: None | Sequence[PathLike] = sys.prefix,
    wsl_virtualenvs: None | Sequence[PathLike] = None,
    sync_requirements: None | Literal["infer"] | PathLike | Sequence[PathLike] = "infer",
    verbosity: Literal[0, 1, 2] = 1,
    extra_args: None | list = None,
) -> None:
    """Run tests. If any test fails, raise an error.

    This is not supposed for normal testing during development. It's usually part of pipeline an runs just
    before pushing code. It usually runs on more python versions and it's syncing dependencies, so takes much
    more time than testing in IDE.

    Args:
        tested_path (PathLike, optional): If None, root is used. root is necessary if using doctest, 'tests'
            folder not works for doctests in modules. Defaults to None.
        tests_path (PathLike, optional): If None, tests is used. It means where venv will be stored etc.
            Defaults to None.
        test_coverage (bool, optional): Whether run test coverage plugin. If True, pytest-cov must
            be installed. Defaults to True.
        stop_on_first_error (bool, optional): Whether stop on first error. Defaults to True.
        virtualenvs (None | Sequence[PathLike], optional): Virtualenvs used to testing. It's used to be able
            to test more python versions at once. Example: ``["venv/37", "venv/310"]``. If you want to use
            current venv, use `sys.prefix`. If there is no venv, it's created with default virtualenv version.
            Defaults to sys.prefix.
        wsl_virtualenvs (None | Sequence[PathLike], optional): If want to test Linux python from windows,
            it's possible with wsl. Just define path to venvs. Beware, that libraries are not synced for wsl
            and that for example python subprocess.run can fails in tests as called from windows.
            Defaults to None
        sync_requirements (None | Literal["infer"] | PathLike | Sequence[PathLike], optional): If using
            `virtualenvs` define what libraries will be installed by path to requirements.txt. Can also be a
            list of more files e.g ``["requirements.txt", "requirements_dev.txt"]``. If "infer", autodetected
            (all requirements). Defaults to "infer".
        verbosity (Literal[0, 1, 2], optional): Whether print details on errors or keep silent. If 0, no
            details, parameters `-q and `--tb=no` are added. if 1, some details are added --tb=short. If 2,
            more details are printed (default --tb=auto) Defaults to 1.
        extra_args (None | list, optional): List of args passed to pytest. Defaults to None

    Raises:
        Exception: If any test fail, it will raise exception (git hook do not continue...).

    Note:
        By default args to quiet mode and no traceback are passed. Usually this just runs automatic tests.
        If some of them fail, it's further analyzed in some other tool in IDE.

    Example:
        ``run_tests(verbosity=2)``
    """
    tested_path = validate_path(tested_path) if tested_path else PROJECT_PATHS.root
    tests_path = validate_path(tests_path) if tests_path else PROJECT_PATHS.tests
    tested_path_str = get_console_str_with_quotes(tested_path)

    verbose = True if verbosity == 2 else False

    if not extra_args:
        extra_args = []

    if not test_coverage:
        pytest_args = [tested_path_str]
    else:
        pytest_args = [
            tested_path_str,
            "--cov",
            get_console_str_with_quotes(PROJECT_PATHS.app),
            "--cov-report",
            get_console_str_with_quotes(f"xml:{tests_path / 'coverage.xml'}"),
        ]

    if stop_on_first_error:
        extra_args.append("-x")

    if verbosity == 0:
        extra_args.append("-q")
        extra_args.append("--tb=no")
    elif verbosity == 1:
        extra_args.append("--tb=short")

    complete_args = [
        "pytest",
        *pytest_args,
        *extra_args,
    ]

    test_command = " ".join(complete_args)

    if (
        sync_requirements
        and sync_requirements != "infer"
        and isinstance(sync_requirements, (Path, str, os.PathLike))
    ):
        sync_requirements = [sync_requirements]
    sync_requirements = cast(list, sync_requirements)

    if virtualenvs:
        test_commands = []
        virtualenvs = [virtualenvs] if isinstance(virtualenvs, (str, Path)) else virtualenvs
        for i in virtualenvs:
            my_venv = venvs.Venv(i)
            my_venv.create()
            if sync_requirements:
                if verbosity:
                    print(f"\tSyncing requirements in venv '{my_venv.venv_path.name}' for tests")
                my_venv.sync_requirements(sync_requirements, verbose)
            # To be able to not install dev requirements in older python venv, pytest is installed.
            # Usually just respond with Requirements already satisfied.
            my_venv.install_library("pytest")
            test_commands.append(f"{my_venv.activate_command} && {test_command}")
    else:
        test_commands = [test_command]

    for i, command in enumerate(test_commands):
        if verbosity and virtualenvs:
            print(f"\tStarting tests with venv `{virtualenvs[i]}`")

        terminal_do_command(
            command, cwd=tested_path.as_posix(), verbose=verbose, error_header="Tests failed."
        )

    if test_coverage:
        delete_files(".coverage")

    if wsl_virtualenvs:
        test_commands = []
        wsl_virtualenvs = [wsl_virtualenvs] if isinstance(wsl_virtualenvs, (str, Path)) else wsl_virtualenvs
        for i in wsl_virtualenvs:
            if verbosity:
                print(f"\nTesting wsl_virtualenv {i}.\n")

            if sync_requirements:
                terminal_do_command(
                    "wsl venv/linux/bin/pip install pytest",
                    verbose=verbose,
                    error_header="Installing libraries in wsl failed.",
                )
                requirements_parsed = get_requirements_files(sync_requirements)
                for j in requirements_parsed:
                    terminal_do_command(
                        f"wsl venv/linux/bin/pip install -r {validate_path(j).relative_to(Path.cwd()).as_posix()}",
                        verbose=verbose,
                        error_header="Installing libraries in wsl failed.",
                    )

            terminal_do_command(
                f"wsl source venv/linux/bin/activate && wsl {i}/bin/pytest",
                cwd=tested_path.as_posix(),
                verbose=verbose,
                error_header="Tests failed.",
            )


def add_readme_tests(readme_path: None | PathLike = None, test_folder_path: None | PathLike = None) -> None:
    """Generate pytest tests script file from README.md and save it to tests folder.

    Can be called from conftest.

    Args:
        readme_path (None | PathLike, optional): If None, autodetected (README.md, Readme.md or readme.md
            on root). Defaults to None.
        test_folder_path (None | PathLike, optional): If None, autodetected (if root / tests).
            Defaults to None.

    Raises:
        FileNotFoundError: If Readme not found.

    Example:
        >>> add_readme_tests()

        Readme tests found.

    Note:
        Only blocks with python defined syntax will be evaluated. Example::

            ```python
            import numpy
            ```

        If you want to import modules and use some global variables, add ``<!--phmdoctest-setup-->`` directive
        before block with setup code.
        If you want to skip some test, add ``<!--phmdoctest-mark.skip-->``
    """
    readme_path = validate_path(readme_path) if readme_path else PROJECT_PATHS.readme
    test_folder_path = validate_path(test_folder_path) if test_folder_path else PROJECT_PATHS.tests

    readme_date_modified = str(readme_path.stat().st_mtime).split(".", maxsplit=1)[0]  # Keep only seconds
    readme_tests_name = f"test_readme_generated-{readme_date_modified}.py"

    test_file_path = test_folder_path / readme_tests_name

    # File not changed from last tests
    if test_file_path.exists():
        return

    for i in test_folder_path.glob("*"):
        if i.name.startswith("test_readme_generated"):
            i.unlink()

    python_path = get_console_str_with_quotes(sys.executable)
    readme = get_console_str_with_quotes(readme_path)
    output = get_console_str_with_quotes(test_file_path)

    generate_readme_test_command = f"{python_path} -m phmdoctest {readme} --outfile {output}"

    terminal_do_command(generate_readme_test_command, error_header="Readme test creation failed")


def deactivate_test_settings() -> None:
    """Deactivate functionality from setup_tests.

    Sometimess you want to run test just in normal mode (enable plots etc.). Usually at the end of
    test file in ``if __name__ = "__main__":`` block.
    """
    mylogging.config.colorize = True

    if "matplotlib" in sys.modules:

        import matplotlib
        from importlib import reload

        reload(matplotlib)



"""Install the package."""
from setuptools import setup, find_packages
import pkg_resources
import re
from pathlib import Path


if __name__ == "__main__":

    with open(Path("requirements.txt").resolve()) as f:
        my_requirements = [str(requirement) for requirement in pkg_resources.parse_requirements(f)]

    with open("README.md") as readme_file:
        readme = readme_file.read()

    with open("mypythontools_cicd/__init__.py") as version_file:
        version = re.findall('__version__ = "(.*)"', version_file.read())[0]

    setup(
        author_email="malachovd@seznam.cz",
        author="Daniel Malachov",
        description="Some tools/functions/snippets used across projects.",
        include_package_data=True,
        install_requires=my_requirements,
        # extras_require={},
        license="mit",
        long_description_content_type="text/markdown",
        long_description=readme,
        name="mypythontools_cicd",
        packages=find_packages(exclude=("tests**",)),
        platforms="any",
        project_urls={
            "Documentation": "https://mypythontools_cicd.readthedocs.io/",
            "Homepage": "https://github.com/Malachov/mypythontools_cicd",
        },
        python_requires=">=3.7",
        url="https://github.com/Malachov/mypythontools_cicd",
        version=version,
        entry_points={
            "console_scripts": [
                "mypythontools_cicd = mypythontools_cicd.project_utils:project_utils_pipeline",
            ],
        },
        classifiers=[
            "Development Status :: 3 - Alpha",
            # "Development Status :: 4 - Beta
            # "Development Status :: 5 - Production/Stable",
            "Environment :: Other Environment",
            "Intended Audience :: Developers",
            "Intended Audience :: Education",
            "License :: OSI Approved :: MIT License",
            "Natural Language :: English",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3.10",
            "Programming Language :: Python :: 3.11",
            "Programming Language :: Python",
            "Topic :: Software Development :: Libraries :: Application Frameworks",
            "Topic :: Software Development :: Libraries :: Python Modules",
        ],
    )



"""Subpackage where you can get paths used in your project (path to README,  __init__.py etc.) or configure
paths to be able other modules here in mypythontools_cicd with PROJECT_PATHS.
"""

from mypythontools_cicd.project_paths.project_paths_internal import (
    PROJECT_PATHS,
    ProjectPaths,
)

__all__ = ["PROJECT_PATHS", "ProjectPaths"]



"""Module with functionality around Continuous Integration and Continuous Delivery.

.. image:: https://img.shields.io/pypi/pyversions/mypythontools_cicd.svg
    :target: https://pypi.python.org/pypi/mypythontools_cicd/
    :alt: Python versions

.. image:: https://badge.fury.io/py/mypythontools_cicd.svg
    :target: https://badge.fury.io/py/mypythontools_cicd
    :alt: PyPI version

.. image:: https://pepy.tech/badge/mypythontools_cicd
    :target: https://pepy.tech/project/mypythontools_cicd
    :alt: Downloads

.. image:: https://img.shields.io/lgtm/grade/python/github/Malachov/mypythontools_cicd.svg
    :target: https://lgtm.com/projects/g/Malachov/mypythontools_cicd/context:python
    :alt: Language grade: Python

.. image:: https://readthedocs.org/projects/mypythontools_cicd/badge/?version=latest
    :target: https://mypythontools_cicd.readthedocs.io/en/latest/?badge=latest
    :alt: Documentation Status

.. image:: https://img.shields.io/badge/License-MIT-yellow.svg
    :target: https://opensource.org/licenses/MIT
    :alt: License: MIT

.. image:: https://codecov.io/gh/Malachov/mypythontools_cicd/branch/master/graph/badge.svg
    :target: https://codecov.io/gh/Malachov/mypythontools_cicd
    :alt: Codecov


Why to use this and not Travis or Circle CI? It's local and it's fast. You can setup it as a task in IDE and
if some phase fails, you know it soon and before pushing to repo.

You can also import mypythontools in your CI/CD and use it there of course.

Links
-----

Official documentation - [readthedocs](https://mypythontools_cicd.readthedocs.io/)

Official repo - [GitHub](https://github.com/Malachov/mypythontools_cicd)


Installation
------------

Python >=3.6 (Python 2 is not supported).

Install with::

    pip install mypythontools_cicd

Subpackages
-----------
Package is divided into several subpackages

:py:mod:`mypythontools_cicd.build`
----------------------------
Build your application to .exe with pyinstaller. It also builds javascript frontend with npm build if configured,
which is used mostly in PyVueEel applications.

:py:mod:`mypythontools_cicd.deploy`
----------------------------
Build package and push it to PyPi,

:py:mod:`mypythontools_cicd.misc`
----------------------------
Miscellaneous functions that are too small to have own subpackage.

:py:mod:`mypythontools_cicd.project_utils`
----------------------------

:py:mod:`mypythontools_cicd.project_paths`
----------------------------
Subpackage where you can get paths used in your project (path to README,  __init__.py etc.).


:py:mod:`mypythontools_cicd.tests`
----------------------------
Runs tests in more venvs with different python versions, also with wsl linux if configured and create coverage.

Mypythontools
==================

There is extra library in separate repository which is not about CICD, but normal python helpers.

https://github.com/Malachov/mypythontools

This can help you with a lot of stuff around CICD like getting project paths, generating docs, testing,
deploying to PyPi etc.

**subpackages**

- config
- misc
- paths
- plots
- property
- terminal
- type_hints
"""
from mypythontools_cicd import build
from mypythontools_cicd import deploy
from mypythontools_cicd import project_paths
from mypythontools_cicd import project_utils
from mypythontools_cicd import tests
from mypythontools_cicd import venvs

__all__ = ["build", "deploy", "project_paths", "project_utils", "tests", "venvs"]

__version__ = "0.0.10"

__author__ = "Daniel Malachov"
__license__ = "MIT"
__email__ = "malachovd@seznam.cz"
